Package: unbalanced
Type: Package
Title: Racing for Unbalanced Methods Selection
Version: 2.0
Date: 2015-06-25
Author: Andrea Dal Pozzolo, Olivier Caelen and Gianluca Bontempi
Maintainer: Andrea Dal Pozzolo <adalpozz@ulb.ac.be>
Description: A dataset is said to be unbalanced when the class of interest (minority class) is much rarer than normal behaviour (majority class). The cost of missing a minority class is typically much higher that missing a majority class. Most learning systems are not prepared to cope with unbalanced data and several techniques have been proposed. This package implements some of most well-known techniques and propose a racing algorithm to select adaptively the most appropriate strategy for a given unbalanced task.
License: GPL (>= 3)
URL: http://mlg.ulb.ac.be
Depends: mlr, foreach, doParallel
Imports: FNN, RANN
Suggests: randomForest, ROCR
Packaged: 2015-06-26 09:54:30 UTC; Andrea
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2015-06-26 13:34:37
